//
//  msgpack_objcTests.m
//  msgpack-objcTests
//
//  Created by akisute on 11/10/15.
//  Copyright (c) 2011年 akisute. All rights reserved.
//

#import "msgpack_objcTests.h"
#import <UIKit/UIKit.h>
#import <OCMock/OCMock.h>

@implementation msgpack_objcTests

- (void)setUp
{
    [super setUp];
}

- (void)tearDown
{
    [super tearDown];
}

- (void)testExample
{
    STAssertTrue(YES, @"Hello, test case!");
}

- (void)testUIKit
{
    // In this SenTestKit unit testing target, we can't use specific UI-related classes and objects
    // Usualy, <UIKit/UIKit.h> is not even imported in templates created by Xcode
    
    // This test should run only in OCUnit, not in GHUnit, since GHUnit can use all UIKit classes without problems
    
    STAssertNil([UIApplication sharedApplication], @"UIApplication is not available");
    STAssertNotNil([UIDevice currentDevice], @"UIDevice is still available");
    STAssertNotNil([UIScreen mainScreen], @"UIScreen is still available, but not sure it returns correct screen sizes");
    STAssertNotNil([UIColor clearColor], @"UIColor is still available");
    //STAssertNotNil([UIFont systemFontOfSize:[UIFont systemFontSize]], @"UIFont is still available, but the test case is halted on this line. You should not use UIFont in the unit test.");
    
    STAssertTrue([[[NSBundle mainBundle] resourcePath] rangeOfString:@"Platforms/iPhoneSimulator.platform"].location != NSNotFound, @"NSBundle is still available, but the bundle resource location (%@) is NOT the application bundle resource location which usually contains resource files. Thus basically we can't use NSBundle in unit tests.", [[NSBundle mainBundle] resourcePath]);
    STAssertNil([UIImage imageNamed:@"SampleImage.png"], @"UIImage is not available (at least with imageNamed: method) because of the NSBundle problem shown above.");
    // Not sure whether UIImage is available through imageWithData:, imageWithContentOfFile: or imageWithCGImage:
    
    // TODO: UIDocument, UIManagedDocument
    // TODO: UINib
    // TODO: UIPrint related classes
    // TODO: UIPasteboard
}

- (void)testOCMock
{
    // Creating a new stub object from class
    {
        id mockedObject = [OCMockObject mockForClass:[NSString class]];
        STAssertThrows([mockedObject length], @"Mocked object raises exception because the fake method is not ready yet.");
        STAssertThrows([mockedObject isKindOfClass:[NSString class]] , @"Mocked object can't even call isKindOfClass: because it's not ready.");
        STAssertFalse([mockedObject class] == [NSString class], @"Mocked object is not a kind of the target class.");
        
        [[[mockedObject stub] andCall:@selector(mockedLength) onObject:self] length];
        STAssertEquals([mockedObject length], (NSUInteger)100, @"Mocked object returns the fake value.");
        
        [[[mockedObject stub] andReturn:@"AllYourBaseAreBelongToUs"] lowercaseString];
        STAssertEqualObjects([mockedObject lowercaseString], @"AllYourBaseAreBelongToUs", @"Returns mocked value.");
        STAssertEquals([mockedObject length], (NSUInteger)100, @"Previously mocked methods are still valid.");
    }
    
    // Method Stubbing for existing object
    {
        NSString *stubTargetObject = @"I am a stub target.";
        STAssertEquals([stubTargetObject length], (NSUInteger)19, @"The original implementation of the length method.");
        
        id mockedObject = [OCMockObject partialMockForObject:stubTargetObject];
        STAssertTrue([mockedObject isKindOfClass:[NSString class]] , @"Mocked object is a kind of the target class.");
        STAssertEquals([mockedObject length], (NSUInteger)19, @"Mocked object returns the original value.");
        
        [[[mockedObject stub] andCall:@selector(mockedLength) onObject:self] length];
        STAssertEquals([mockedObject length], (NSUInteger)100, @"Mocked object returns the fake value.");
        STAssertEquals([stubTargetObject length], (NSUInteger)100, @"Stubbed target object also returns the fake value.");
    }
}
- (NSUInteger)mockedLength
{
    return 100;
}

@end
